package com.loafle.overflow.central.module.target.service;

import java.util.List;

import com.loafle.overflow.central.module.infra.service.CentralInfraHostService;
import com.loafle.overflow.central.module.probe.service.CentralProbeService;
import com.loafle.overflow.central.module.target.dao.TargetDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.discovery.Host;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.model.target.Target;
import com.loafle.overflow.service.central.target.TargetService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Created by insanity on 17. 6. 28.
 */
@Service("TargetService")
public class CentralTargetService implements TargetService {

    @Autowired
    private TargetDAO targetDAO;
    @Autowired
    private CentralProbeService probeService;
    @Autowired
    private CentralInfraHostService infraHostService;

    @Transactional
    public Target regist(Target target, Probe probe) throws OverflowException {
        this.probeService.increaseTargetCount(probe);
        return this.targetDAO.save(target);
    }

    @Transactional
    public void remove(Target target, Probe probe) throws OverflowException {
        this.probeService.decreaseTargetCount(probe);
        this.targetDAO.delete(target);
    }

    public Target read(String id) throws OverflowException {
        return this.targetDAO.findOne(Long.valueOf(id));
    }

    public Target modify(Target target) throws OverflowException {
        return this.targetDAO.save(target);
    }

    @Deprecated
    public Target regist(Target target) throws OverflowException {
        return this.targetDAO.save(target);
    }

    @Deprecated
    public void remove(Target target) throws OverflowException {
        this.targetDAO.delete(target);
    }

    public Target increaseSensorCount(Target target) throws OverflowException {
        Target t = this.targetDAO.findOne(target.getId());
        t.setSensorCount(t.getSensorCount() + 1);
        return this.targetDAO.save(t);
    }

    public Target decreaseSensorCount(Target target) throws OverflowException {
        Target t = this.targetDAO.findOne(target.getId());
        int count = t.getSensorCount();
        if (t.getSensorCount() > 0) {
            t.setSensorCount(count - 1);
        }
        return this.targetDAO.save(t);
    }

    @Transactional
    public List<Target> registDiscoveredTargets(Long probeId, List<Host> hosts,
            List<com.loafle.overflow.model.discovery.Service> services) throws OverflowException {
        /*
         * 해당 host의 mac에 해당하는 infraHost가 이미 존재하는지 검사 infraHost가 없다면 create infraHost가
         * 있다면 update infraHost.getTarget()이 있어도 무조건 새로운 target으로 연결
         * 
         * 
         * for(Host host : hosts) { Target target = new Target();
         * 
         * MetaInfraType infraType = new MetaInfraType(); infraType.setId(2); InfraHost
         * infraHost = new InfraHost(); infraHost.setInfraType(infraType);
         * infraHost.setProbe(new Probe(probeId)); infraHost.setTarget(target); }
         */
        return null;
    }
}
